/*
 * PjViewer_IP.java
 *
 * Created on December 9, 2004, 6:49 PM
 */

package Grapher2D.Viewer;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.*;
import java.awt.datatransfer.*;

import java.lang.SecurityManager;
import jv.objectGui.PsTabPanel;
import jv.object.PsConfig;
import jv.object.PsDebug;
import jv.object.PsObject;
import jv.object.PsPanel;
import jv.object.PsUpdateIf;
import jv.project.PvDisplayIf;
import jv.project.PgGeometryIf;
import jv.project.PjProject_IP;
import Grapher2D.geom.*;
import Grapher2D.util.*;
import jv.geom.PgPointSet;
import jv.project.PgGeometry;

/**
 * Info panel for PjViewer 
 * 
 * @author		David Lippman
 * @version		Nov 25, 2004
 */
public class PjViewer_IP extends PjProject_IP implements ItemListener,ActionListener {
        public final static int TYPE_FUNC = 1; /**f(x,y) type*/
        public final static int TYPE_PIECE = 2; /**Parametric Surface type*/   
        public final static int TYPE_PCURVE = 3; /**Parametric Curve type*/
        public final static int TYPE_IMPSURF= 4; /**Implicit Surface type*/
        public final static int TYPE_VF= 5; /**Vector Field on grid type*/
        public final static int TYPE_VFOS = 6; /**Vector Field on surface type*/
        public final static int TYPE_IMPPOLAR = 7; /**Implicit Polar type*/
         
        
        protected	PjViewer			m_PjViewer;
        protected       Choice                          m_selector;
        protected       Choice                          m_cNew;
        protected       Checkbox                        m_cClip;
        
        protected       PsTabPanel                      m_tabpanel;
        protected       PsPanel                         m_pMain;
        protected       PsPanel                         m_pSel;
        protected       PsPanel                         cps;
        protected       PsPanel                         cp;
        protected       PsPanel                         m_pBnds;
        protected       Button                          m_bZoomFit;
        protected       Button                          m_bRemove;
        protected       Button                          m_bToClipboard;
        protected       boolean                         m_bCanUseClipboard = false;
        
        protected       Button                          m_bGetClip;
        protected       Checkbox                         m_bSclAxes;
        protected       Checkbox                        m_bShowGrid;
        protected       Button                          m_bSetCenter;
        protected       String[]                        m_typenames = {"Choose a graph to add",
                                                                        "Parametric Curve",
                                                                        "y=f(x) Curve",
                                                                        "Implicit Function",
                                                                        "Polar Function",
                                                                        "Piecewise Function",
                                                                        "Vector Field",
                                                                        "Vector Field on Selected"};
       protected        int[]                           m_typetable = {0, 
                                                                        TYPE_PCURVE, 
                                                                        TYPE_FUNC,
                                                                        TYPE_IMPSURF,
                                                                        TYPE_IMPPOLAR,
                                                                        TYPE_PIECE,
                                                                        TYPE_VF,
                                                                        TYPE_VFOS};
       protected        String[]                        m_typeshortnames = {
                                          "None","PCurve", "Func", "ImpCurve", "Polar", "PW", "VecFld","VF on"};            
       
        
	public PjViewer_IP() {
		super();
		if (getClass() == PjViewer_IP.class)
			init();
	}
	public void init() {
                super.init();
                
                m_tabpanel = new PsTabPanel();
                add(m_tabpanel);
                m_tabpanel.init();
                
                m_pMain = new PsPanel(new BorderLayout());
                
                m_pSel = new PsPanel(new GridLayout(2, 2));
                m_pSel.add(new Label("New graph:"));
                m_cNew = new Choice();
                m_cNew.addItemListener(this);
  
                for (int i = 0; i< m_typenames.length; i++) {
                    m_cNew.add(m_typenames[i]);
                }
/*                m_cNew.add("Choose a graph to add");
                m_cNew.add("Parametric Curve");
                m_cNew.add("Parametric Surface");
                m_cNew.add("z=f(x,y) Surface");
                m_cNew.add("Vector Field");
                m_cNew.add("Vector Field on selected");
                m_cNew.add("Implicit Surface");
*/                                
                m_pSel.add(m_cNew);
                m_pSel.add(new Label("Select Existing:"));
                m_selector = new Choice();
                m_selector.addItemListener(this);
                m_pSel.add(m_selector);
                m_pMain.add(m_pSel, BorderLayout.NORTH);
                
             
                cps = new PsPanel(new GridLayout(1, 1));
                cps.setName("cps");
                cps.setVisible(true);
                cps.setBorderType(PsPanel.BORDER_NONE);
                m_pMain.add(cps, BorderLayout.CENTER);
                
                m_pBnds = new PsPanel();
                
                
                PsPanel pBottomButtons = new PsPanel(new GridLayout(1,2));
//TODO: Reimplement Get Bounds from Geometry to do real zoomfit
//                m_bZoomFit = new Button ("Zoom Fit");
//                m_bZoomFit.addActionListener(this);
//                pBottomButtons.add(m_bZoomFit);
                m_bRemove = new Button ("Remove Selected");
                m_bRemove.addActionListener(this);
                pBottomButtons.add(m_bRemove);
                
                try {
                  
                    SecurityManager sm = System.getSecurityManager();
                    if (sm != null) sm.checkSystemClipboardAccess();
                    
                    m_bCanUseClipboard = true;
                    m_bToClipboard = new Button ("Copy to Clipboard");
                    m_bToClipboard.addActionListener(this);
                    pBottomButtons.add(m_bToClipboard);
                }
                catch (SecurityException e) {
                     
                   }
/*  Use when reimplement zoom fit		
                if (m_bCanUseClipboard) {
                    pBottomButtons.setLayout(new GridLayout(2,2));
                    pBottomButtons.add(m_bZoomFit);
                    pBottomButtons.add(m_bRemove);
                    pBottomButtons.add(m_bToClipboard);
                } else {
                    pBottomButtons.setLayout(new GridLayout(1,2));
                    pBottomButtons.add(m_bZoomFit);
                    pBottomButtons.add(m_bRemove);
 
                }
*/                
		m_pMain.add(pBottomButtons, BorderLayout.SOUTH);
		
		
		addTitle("");
	}
	public void setParent(PsUpdateIf parent) {
		super.setParent(parent);
		m_PjViewer = (PjViewer)parent;
                
 //               m_pBnds.add(m_PjViewer.xscl.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
 //              m_pBnds.add(m_PjViewer.yscl.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
 //               m_pBnds.add(m_PjViewer.zscl.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
                
 //               m_cClip = new Checkbox("Use Clip Bounds");
 //               m_cClip.addItemListener(this);
 //               m_pBnds.add(m_cClip);
                
               m_pBnds.add(m_PjViewer.m_clipbounds.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
   //            m_pBnds.add(m_PjViewer.xint.assureInspector(PsPanel.INFO,PsPanel.INFO_EXT));
//               m_bGetClip = new Button ("Get bounds from current Geom");
//                m_bGetClip.addActionListener(this);
//                m_pBnds.add(m_bGetClip);
                m_bSclAxes = new Checkbox ("Scale Axes to current Bounds");
                m_bSclAxes.addItemListener(this);
                m_pBnds.add(m_bSclAxes);
                m_bShowGrid = new Checkbox ("Show XY Grid", true);
                m_bShowGrid.addItemListener(this);
                m_pBnds.add(m_bShowGrid);
//Removed cuz it doesn't do anything in 2D.  Need to reimplement getGeomBnds
                //                m_bSetCenter = new Button ("Center Camera on current Bounds");
//                m_bSetCenter.addActionListener(this);
//                m_pBnds.add(m_bSetCenter);
               
               
                
                m_tabpanel.addPanel("Graphs",m_pMain);
                m_tabpanel.addPanel("Window",m_pBnds);
                m_tabpanel.validate();
   
	}
        
        public void changeCP(String gname) {
                cps.removeAll();
                
                PgGeometry newgraph =  (PgGeometry) m_PjViewer.m_graphs.get(gname);
 //               cp = newgraph.assureInspector(PsPanel.CONFIG,PsPanel.CONFIG_EXT);
 //                
 //               cp.setBorderType(PsPanel.BORDER_NONE);
 //               
 //               cps.add(cp);
//                
 //               update(cp);
                
//                update(cps);
                cps.add(newgraph.assureInspector(PsPanel.CONFIG,PsPanel.CONFIG_EXT));

                validate();
//                m_PjViewer.update(m_PjViewer);
                
                
                
        }
        
        public void itemStateChanged(ItemEvent event) {
		if (m_project==null)
			return;
		Object source = event.getSource();
                if (source == m_selector){
                        changeCP(m_selector.getSelectedItem());
                         return;
                }
                if (source == m_cNew) {
                    int sel = m_cNew.getSelectedIndex();
                    if (sel == 0) { return;}
                    
                    else {
                        String graphname;
                        boolean succ = false;
                        if (m_typetable[sel]==TYPE_VFOS) {
                            if (m_selector.getSelectedIndex() == -1) {return;}
                            String selected = m_selector.getSelectedItem();
                            graphname = "VF on " + selected;
                            if (selected.startsWith("VF")) {
                                succ = false;
                            } else {
                                
                                m_PjViewer.addGraph(m_selector.getSelectedItem(),m_typetable[sel]);
                                succ = true;
                            }
                        } else {
                            graphname = m_typeshortnames[sel] + Integer.toString(m_PjViewer.m_gDef);
                            m_PjViewer.addGraph(graphname, m_typetable[sel]);
                            succ = true;
                        }
                        
                        if (succ) {
                            int res = m_selector.getItemCount();
                            m_selector.add(graphname);
                            m_selector.select(res);
                            changeCP(graphname);
                            m_PjViewer.setChildDomains();
                        }
                        m_cNew.select(0);
                    }
                    return;    
                   
                }
                if (source == m_bSclAxes){
                    if (m_bSclAxes.getState()) {
                        m_PjViewer.setScl();
                        m_PjViewer.update(m_PjViewer);
                    } else {
                        m_PjViewer.unsetScl();
                    }
                }
              
                if (source == m_cClip) {
                    m_PjViewer.setClip(m_cClip.getState());
                }
                if (source == m_bShowGrid) {
                    if (m_bShowGrid.getState()) {
                        m_PjViewer.addGeometry(m_PjViewer.m_blackboard);
                    } else {
                        m_PjViewer.removeGeometry(m_PjViewer.m_blackboard);
                    }
                }
                
        }
        
        public void actionPerformed(ActionEvent event) {
		
		Object source = event.getSource();
/*
                if (source == m_bZoomFit) {
                    m_PjViewer.setCamCenter();
                    //                        m_PjViewer.fitDisplays();
//                        m_PjViewer.update(m_PjViewer);
                }
*/
                 if (source == m_bRemove) {
                    if (m_selector.getItemCount() > 0) {
                        boolean hasvf = m_PjViewer.removeGraph(m_selector.getSelectedItem());
                        
                        if (hasvf) {
                            m_selector.remove("VF on " + m_selector.getSelectedItem());
                        }
                        m_selector.remove(m_selector.getSelectedIndex());
                        
                        if (m_selector.getItemCount()>0){
                            m_selector.select(0);
                            changeCP(m_selector.getSelectedItem());
                        } else {
                             cps.removeAll();
                        }
                    }   
                }
                if (source == m_bGetClip) {
 /*                   PjMulti curgraph = (PjMulti) m_PjViewer.m_graphs.get(m_selector.getSelectedItem());
                    double[] bds = curgraph.getBnds();
                    m_PjViewer.xmin.setValue(bds[0]);
                    m_PjViewer.xmax.setValue(bds[1]);
                    m_PjViewer.ymin.setValue(bds[2]);
                    m_PjViewer.ymax.setValue(bds[3]);
                    m_PjViewer.zmin.setValue(bds[4]);
                    m_PjViewer.zmax.setValue(bds[5]);
                    m_PjViewer.updateClip();
                    m_PjViewer.update(m_PjViewer);
   */             }
                if (source == m_bToClipboard) {
                    PvDisplayIf disp = m_PjViewer.getDisplay();
                    ImageSelection contents = new ImageSelection(disp.getImage());
                    Clipboard systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                    systemClipboard.setContents(contents, null);
                    

//                      ImageViewer iviewer = new ImageViewer(disp.getImage());
                }
/*                
                if (source == m_bSetCenter) {
                    
                    m_PjViewer.setCamCenter();
                    m_PjViewer.update(m_PjViewer);
                }
*/               
	}
        
     
        
}
