/*
 * PgImpSurf.java
 *
 * Created on December 5, 2004, 6:19 PM
 */

package Grapher2D.geom;

import java.awt.Color;
import jv.function.PuFunction;
import jv.number.PuDouble;
import jv.object.PsPanel;
import jv.geom.PgElementSet;
import jv.number.PdColor;
import jv.number.PuInteger;
import jv.object.PsDebug;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
import Grapher2D.util.PuDomain;
/**
 *
 * @author David
 */
public class PgImpSurf extends PgElementSet {

    public final static int TYPE_IMPSURF= 4; /**Implicit Surface type*/
    public final static int TYPE_IMPPOLAR = 7; /**Implicit Polar type*/
 
    protected       PuFunction                      m_func2;
    protected       PuDomain                        m_domain;
    protected       int                             coords;
    protected       int[][][]                       xedge;
    protected       int[][]                         yedge;
    protected       int[]                           zedge;
    protected       double[][][]                      xpoint;
    protected       double[][]                        ypoint;
    protected       double[]                          zpoint;
    protected       double[][]                      xval;
    protected       double[]                        yval;
    protected       double                          zval;
    protected       int[]                           discr;
    protected       boolean                         m_bHasVf = false;  
    protected       boolean                         m_bUseGlobalDomain = false;
    protected       double[]                        globaldomain;
    
static final int[][] triTable = {{-1},
{3,0,8,-1},
{0,1,9,-1},
{3,1,9,8,-1},
{1,2,10,-1},
{3,2,10,1,0,8,-1},
{0,2,10,9,-1},
{3,2,10,9,8,-1},
{2,3,11,-1},
{11,2,0,8,-1},
{2,1,9,0,3,11,-1},
{11,2,1,9,8,-1},
{10,1,3,11,-1},
{11,10,1,0,8,-1},
{10,9,0,3,11,-1},
{11,10,9,8,-1},
{4,7,8,-1},
{0,4,7,3,-1},
{0,1,9,4,7,8,-1},
{1,9,4,7,3,-1},
{4,7,8,-1,1,2,10,-1},
{2,10,1,0,4,7,3,-1},
{0,2,10,9,4,7,8,-1},
{2,10,9,4,7,3,-1},
{4,7,11,2,3,8,-1},
{2,0,4,7,11,-1},
{0,3,8,-1,2,1,9,4,7,11,-1},
{2,1,9,4,7,11,-1},
{4,7,11,10,1,3,8,-1},
{10,1,0,4,7,11,-1},
{0,3,8,-1,10,9,4,7,11,-1},
{10,9,4,7,11,-1},
{5,4,9,-1},
{3,0,9,5,4,8,-1},
{5,4,0,1,-1},
{3,1,5,4,8,-1},
{1,2,10,5,4,9,-1},
{3,2,10,5,4,8,-1,1,0,9,-1},
{5,4,0,2,10,-1},
{3,2,10,5,4,8,-1},
{2,3,11,-1,5,4,9,-1},
{11,2,0,9,5,4,8,-1},
{2,1,5,4,0,3,11,-1},
{11,2,1,5,4,8,-1},
{10,5,4,9,1,3,11,-1},
{11,10,5,4,8,-1,1,0,9,-1},
{10,5,4,0,3,11,-1},
{11,10,5,4,8,-1},
{9,5,7,8,-1},
{0,9,5,7,3,-1},
{0,1,5,7,8,-1},
{1,5,7,3,-1},
{9,1,2,10,5,7,8,-1},
{2,10,5,7,3,-1,1,0,9,-1},
{0,2,10,5,7,8,-1},
{2,10,5,7,3,-1},
{9,5,7,11,2,3,8,-1},
{2,0,9,5,7,11,-1},
{0,3,8,-1,2,1,5,7,11,-1},
{2,1,5,7,11,-1},
{9,1,3,8,-1,10,5,7,11,-1},
{10,5,7,11,-1,1,0,9,-1},
{0,3,8,-1,10,5,7,11,-1},
{10,5,7,11,-1},
{6,5,10,-1},
{3,0,8,-1,6,5,10,-1},
{0,1,10,6,5,9,-1},
{3,1,10,6,5,9,8,-1},
{2,6,5,1,-1},
{3,2,6,5,1,0,8,-1},
{0,2,6,5,9,-1},
{3,2,6,5,9,8,-1},
{6,5,10,2,3,11,-1},
{11,6,5,10,2,0,8,-1},
{6,5,9,0,3,11,-1,2,1,10,-1},
{11,6,5,9,8,-1,2,1,10,-1},
{6,5,1,3,11,-1},
{11,6,5,1,0,8,-1},
{6,5,9,0,3,11,-1},
{11,6,5,9,8,-1},
{4,5,10,6,7,8,-1},
{0,4,5,10,6,7,3,-1},
{0,1,10,6,7,8,-1,4,5,9,-1},
{1,10,6,7,3,-1,4,5,9,-1},
{4,5,1,2,6,7,8,-1},
{2,6,7,3,-1,0,4,5,1,-1},
{0,2,6,7,8,-1,4,5,9,-1},
{2,6,7,3,-1,4,5,9,-1},
{4,5,10,2,3,8,-1,6,7,11,-1},
{6,7,11,-1,2,0,4,5,10,-1},
{0,3,8,-1,6,7,11,-1,4,5,9,-1,2,1,10,-1},
{6,7,11,-1,4,5,9,-1,2,1,10,-1},
{4,5,1,3,8,-1,6,7,11,-1},
{6,7,11,-1,0,4,5,1,-1},
{0,3,8,-1,6,7,11,-1,4,5,9,-1},
{6,7,11,-1,4,5,9,-1},
{10,6,4,9,-1},
{3,0,9,10,6,4,8,-1},
{6,4,0,1,10,-1},
{3,1,10,6,4,8,-1},
{1,2,6,4,9,-1},
{3,2,6,4,8,-1,1,0,9,-1},
{2,6,4,0,-1},
{3,2,6,4,8,-1},
{6,4,9,10,2,3,11,-1},
{11,6,4,8,-1,10,2,0,9,-1},
{6,4,0,3,11,-1,2,1,10,-1},
{11,6,4,8,-1,2,1,10,-1},
{6,4,9,1,3,11,-1},
{11,6,4,8,-1,1,0,9,-1},
{6,4,0,3,11,-1},
{11,6,4,8,-1},
{9,10,6,7,8,-1},
{0,9,10,6,7,3,-1},
{0,1,10,6,7,8,-1},
{1,10,6,7,3,-1},
{9,1,2,6,7,8,-1},
{2,6,7,3,-1,1,0,9,-1},
{0,2,6,7,8,-1},
{2,6,7,3,-1},
{9,10,2,3,8,-1,6,7,11,-1},
{6,7,11,-1,10,2,0,9,-1},
{0,3,8,-1,6,7,11,-1,2,1,10,-1},
{6,7,11,-1,2,1,10,-1},
{9,1,3,8,-1,6,7,11,-1},
{6,7,11,-1,1,0,9,-1},
{0,3,8,-1,6,7,11,-1},
{6,7,11,-1},
{7,6,11,-1},
{7,6,11,3,0,8,-1},
{7,6,11,-1,0,1,9,-1},
{7,6,11,3,1,9,8,-1},
{7,6,10,1,2,11,-1},
{7,6,10,1,0,8,-1,3,2,11,-1},
{7,6,10,9,0,2,11,-1},
{7,6,10,9,8,-1,3,2,11,-1},
{7,6,2,3,-1},
{7,6,2,0,8,-1},
{7,6,2,1,9,0,3,-1},
{7,6,2,1,9,8,-1},
{7,6,10,1,3,-1},
{7,6,10,1,0,8,-1},
{7,6,10,9,0,3,-1},
{7,6,10,9,8,-1},
{4,6,11,8,-1},
{3,0,4,6,11,-1},
{0,1,9,4,6,11,8,-1},
{3,1,9,4,6,11,-1},
{4,6,10,1,2,11,8,-1},
{3,2,11,-1,1,0,4,6,10,-1},
{0,2,11,8,-1,4,6,10,9,-1},
{3,2,11,-1,4,6,10,9,-1},
{4,6,2,3,8,-1},
{4,6,2,0,-1},
{0,3,8,-1,4,6,2,1,9,-1},
{4,6,2,1,9,-1},
{4,6,10,1,3,8,-1},
{1,0,4,6,10,-1},
{0,3,8,-1,4,6,10,9,-1},
{4,6,10,9,-1},
{7,4,9,5,6,11,-1},
{7,4,8,-1,3,0,9,5,6,11,-1},
{7,4,0,1,5,6,11,-1},
{7,4,8,-1,3,1,5,6,11,-1},
{7,4,9,1,2,11,-1,5,6,10,-1},
{7,4,8,-1,3,2,11,-1,1,0,9,-1,5,6,10,-1},
{7,4,0,2,11,-1,5,6,10,-1},
{7,4,8,-1,3,2,11,-1,5,6,10,-1},
{7,4,9,5,6,2,3,-1},
{7,4,8,-1,5,6,2,0,9,-1},
{7,4,0,3,-1,5,6,2,1,-1},
{7,4,8,-1,5,6,2,1,-1},
{7,4,9,1,3,-1,5,6,10,-1},
{7,4,8,-1,1,0,9,-1,5,6,10,-1},
{7,4,0,3,-1,5,6,10,-1},
{7,4,8,-1,5,6,10,-1},
{9,5,6,11,8,-1},
{3,0,9,5,6,11,-1},
{0,1,5,6,11,8,-1},
{3,1,5,6,11,-1},
{9,1,2,11,8,-1,5,6,10,-1},
{3,2,11,-1,1,0,9,-1,5,6,10,-1},
{0,2,11,8,-1,5,6,10,-1},
{3,2,11,-1,5,6,10,-1},
{9,5,6,2,3,8,-1},
{5,6,2,0,9,-1},
{0,3,8,-1,5,6,2,1,-1},
{5,6,2,1,-1},
{9,1,3,8,-1,5,6,10,-1},
{1,0,9,-1,5,6,10,-1},
{0,3,8,-1,5,6,10,-1},
{5,6,10,-1},
{7,5,10,11,-1},
{7,5,10,11,3,0,8,-1},
{7,5,9,0,1,10,11,-1},
{7,5,9,8,-1,3,1,10,11,-1},
{7,5,1,2,11,-1},
{7,5,1,0,8,-1,3,2,11,-1},
{7,5,9,0,2,11,-1},
{7,5,9,8,-1,3,2,11,-1},
{7,5,10,2,3,-1},
{7,5,10,2,0,8,-1},
{7,5,9,0,3,-1,2,1,10,-1},
{7,5,9,8,-1,2,1,10,-1},
{7,5,1,3,-1},
{7,5,1,0,8,-1},
{7,5,9,0,3,-1},
{7,5,9,8,-1},
{4,5,10,11,8,-1},
{3,0,4,5,10,11,-1},
{0,1,10,11,8,-1,4,5,9,-1},
{3,1,10,11,-1,4,5,9,-1},
{4,5,1,2,11,8,-1},
{3,2,11,-1,0,4,5,1,-1},
{0,2,11,8,-1,4,5,9,-1},
{3,2,11,-1,4,5,9,-1},
{4,5,10,2,3,8,-1},
{2,0,4,5,10,-1},
{0,3,8,-1,4,5,9,-1,2,1,10,-1},
{4,5,9,-1,2,1,10,-1},
{4,5,1,3,8,-1},
{0,4,5,1,-1},
{0,3,8,-1,4,5,9,-1},
{4,5,9,-1},
{7,4,9,10,11,-1},
{7,4,8,-1,3,0,9,10,11,-1},
{7,4,0,1,10,11,-1},
{7,4,8,-1,3,1,10,11,-1},
{7,4,9,1,2,11,-1},
{7,4,8,-1,3,2,11,-1,1,0,9,-1},
{7,4,0,2,11,-1},
{7,4,8,-1,3,2,11,-1},
{7,4,9,10,2,3,-1},
{7,4,8,-1,10,2,0,9,-1},
{7,4,0,3,-1,2,1,10,-1},
{7,4,8,-1,2,1,10,-1},
{7,4,9,1,3,-1},
{7,4,8,-1,1,0,9,-1},
{7,4,0,3,-1},
{7,4,8,-1},
{9,10,11,8,-1},
{3,0,9,10,11,-1},
{0,1,10,11,8,-1},
{3,1,10,11,-1},
{9,1,2,11,8,-1},
{3,2,11,-1,1,0,9,-1},
{0,2,11,8,-1},
{3,2,11,-1},
{9,10,2,3,8,-1},
{10,2,0,9,-1},
{0,3,8,-1,2,1,10,-1},
{2,1,10,-1},
{9,1,3,8,-1},
{1,0,9,-1},
{0,3,8,-1},
{-1}};
    
    
    /** Creates a new instance of PgImpSurf */
    public PgImpSurf(String Pname, int type) {
        super(3);
        if (getClass() == PgImpSurf.class) {
            init(Pname, type);
        }
    }
    
    public void init(String Pname, int type) {
        super.init();
        setName(Pname);
        
        coords = type;
        
        m_func2 = new PuFunction(3,1);
        m_func2.setName("Functions");
        m_domain = new PuDomain();
        
        if (coords == TYPE_IMPSURF) {
            m_func2.setVariables(new String[] {"x","y","z"});
            m_domain.setVars(new String[] {"x","y"});
        } else if (coords == TYPE_IMPPOLAR) {
            m_func2.setVariables(new String[] {"r","t","z"});
            m_domain.setVars(new String[] {"r","t"});
            m_domain.setDefVals(new double[] {0.0,5.0,0.0,2*Math.PI});
        }
        m_func2.setExpression(0,"0");
        
        m_func2.setParent(this);
        m_domain.setDefDiscr(new int[] {40});
        m_domain.setNumDiscr(1);
        m_domain.init();
        m_domain.setParent(this);
       
//        assureElementColors();
 //       showElementColors(true);
        showVertices(false);
        showElements(false);
        setGlobalEdgeColor(Color.black);
//        computeImpSurf();
        
    }
    
     public void start() {
		if (PsDebug.NOTIFY) PsDebug.notify("start: ");
    }

    public boolean update(Object event) {
		if (event == this) {
                        computeImpSurf();
			return super.update(this);
		} else if (event == m_domain) {
                    update(this);
                    
                } else if (event == m_func2) {
                    update(this);
                }
                return true;
    }
    
     public void computeImpSurf() {
    
            super.init();
            Polygonize();
            assureElementColors();
//            makeNeighbour();
//            makeElementNormals();
//	    makeElementColorsFromXYZ();
//            showElementColors(true);
            
        }
        
        public void Polygonize () {
            
            discr = new int[3];
            discr[0] = m_domain.getDiscr(0);
            discr[1] = discr[0];
            discr[2] = 1;
            double xmin = m_domain.getMin(0);
            double ymin = m_domain.getMin(1);
            double zmin = 0;
            double deltaX = (m_domain.getMax(0)-xmin)/discr[0];
            double deltaY = (m_domain.getMax(1)-ymin)/discr[1];
            double deltaZ = .1;
 //           double xmax = m_domain.getMax(0);
 //           double ymax = m_domain.getMax(1);
 //           double zmax = m_domain.getMax(2);
            double x,y,z;
            double isolevel = 0.0;
            

            
            xedge = new int[discr[1]+1][discr[2]+1][2];
            yedge = new int[discr[2]+1][2];
            zedge = new int[2];
            
            xpoint = new double[discr[1]+1][discr[2]+1][3];
            ypoint = new double[discr[2]+1][3];
            zpoint = new double[3];
            xval = new double[discr[1]+1][discr[2]+1];
            yval = new double[discr[2]+1];
            
            double[][] cubepoints;
            cubepoints = new double[8][3];
            double[] cubevals;
            cubevals = new double[8];
           
    
  //Run cubes        
            initXstorage(discr, xmin,ymin,zmin,deltaX, deltaY, deltaZ, isolevel);
            x = xmin;
            for (int i=0; i < discr[0]; i++)  {
                initYstorage(discr, x,ymin,zmin,deltaX, deltaY, deltaZ, isolevel);
                y = ymin;
                for (int j=0; j < discr[1]; j++) {
                    initZstorage(discr, x,y,zmin,deltaX, deltaY, deltaZ, isolevel);
                    z = zmin;
                    for (int k=0; k < discr[2]; k++) {
                        
                        
                        cubepoints[6]= new double[] {x+deltaX,y+deltaY,z+deltaZ};
                        cubevals[6] = m_func2.eval(0,cubepoints[6]);
                       
                        Polygoner(cubepoints[6],cubevals[6],i,j,k);
                        z += deltaZ;
                    }
                    y += deltaY;
                }
                x += deltaX;
            }
            
            
        }
        
        
        public int Polygoner(double[] pt, double val, int xloc, int yloc, int zloc) {
        double isolevel = 0.0;
        int i;
        int cubeindex;
        int[] vertlist;
//        double[][] vertlist;

  //      vertlist = new double[12][3];
        vertlist = new int[12];

  /*
      Determine the index into the edge table which
      tells us which vertices are inside of the surface
   */
   cubeindex = 0;
   if (val < isolevel)  { cubeindex |= 64; }
   
   if ((val < isolevel) != (zval < isolevel)) {
       vertlist[10] =
         addVertex(new PdVector( VertexInterp(isolevel,zpoint,pt,zval,val)));
       if ((cubeindex & 64) == 0)  { cubeindex |= 4; }
   } else { 
       vertlist[10] = -1; 
       if ((cubeindex & 64) != 0)  { cubeindex |= 4; }
   }
      
   if ((val < isolevel) != (yval[zloc] < isolevel)) {
       vertlist[5] =
         addVertex(new PdVector( VertexInterp(isolevel,ypoint[zloc],pt,yval[zloc],val)));
       if ((cubeindex & 64) == 0)  { cubeindex |= 32; }
   } else { 
       vertlist[5] = -1; 
       if ((cubeindex & 64) != 0)  { cubeindex |= 32; }
   }
   
   if ((val < isolevel) != (xval[yloc][zloc] < isolevel)) {
       vertlist[6] =
         addVertex(new PdVector( VertexInterp(isolevel,pt,xpoint[yloc][zloc],val,xval[yloc][zloc])));
       if ((cubeindex & 64) == 0)  { cubeindex |= 128; }
   } else { 
       vertlist[6] = -1; 
       if ((cubeindex & 64) != 0)  { cubeindex |= 128; }
   }
   
      /* Find the vertices where the surface intersects the cube */
  vertlist[0] = yedge[zloc][0];
  vertlist[1] = zedge[0];
  vertlist[2] = zedge[1];
  vertlist[3] = xedge[yloc][zloc][0];
  vertlist[4] = yedge[zloc+1][0];
  vertlist[7] = xedge[yloc][zloc+1][0];
  vertlist[8] = xedge[yloc][zloc][1];
  vertlist[9] = yedge[zloc][1];
  vertlist[11] = xedge[yloc+1][zloc][1];
   
   // follow tree to set cubeindex of rest of points
   if (vertlist[1] != -1) { // pt on eg1
       if ((cubeindex & 4) == 0) { cubeindex |= 2;}
   } else { if ((cubeindex & 4) != 0) { cubeindex |= 2;}}
   if (vertlist[2] != -1) { // pt on eg2
       if ((cubeindex & 4) == 0) { cubeindex |= 8;}
   } else { if ((cubeindex & 4) != 0) { cubeindex |= 8;}} 
   if (vertlist[0] != -1) { // pt on eg0
       if ((cubeindex & 2) == 0) { cubeindex |= 1;}
   } else {  if ((cubeindex & 2) != 0) { cubeindex |= 1;}}
   if (vertlist[8] != -1) { // pt on eg8
       if ((cubeindex & 1) == 0) { cubeindex |= 16;}
   } else { if ((cubeindex & 1) != 0) { cubeindex |= 16;} }
  
 
   /* Create the element */
 
   int vcount = 0;
   int lastloc = 0;
   for (i=0;i<triTable[cubeindex].length ;i++) {
       if (triTable[cubeindex][i] != -1) {
           vcount++;
       } else {
           if (vcount > 0) {
               int[] verts = new int[vcount];
               boolean badcube = false;
               for (int k=0;k<vcount;k++) {
                     verts[vcount - k - 1] = vertlist[triTable[cubeindex][k+lastloc]];
                     if (verts[vcount-k-1] == -1) { badcube = true;}
//                   verts[k] = addVertex(new PdVector(vertlist[triTable[cubeindex][k+lastloc]]));                 
               }
              if (badcube == false) {
               addElement(new PiVector(verts));
              } else {
                   System.out.println("cubeindex: " + cubeindex);
                   System.out.println("Location: " + xloc +"," + yloc + "," + zloc);
                   for (int k=0; k<vcount;k++) {
                       System.out.println(triTable[cubeindex][k+lastloc] + ": " +verts[vcount-k-1]);
                   
                   }
                   
              }
               vcount = 0;
               lastloc = i+1;
           }
       }
   }
   
   /* Do subs to update storage*/
    xedge[yloc][zloc][0] = vertlist[1];
    xedge[yloc][zloc][1] = vertlist[9];
    yedge[zloc][0] = vertlist[2];
    yedge[zloc][1] = vertlist[10];
    zedge[0] = vertlist[5];
    zedge[1] = vertlist[6];
    xpoint[yloc][zloc] = pt;
    xval[yloc][zloc] = val;
    ypoint[zloc] = pt;
    yval[zloc] = val;
    zpoint = pt;
    zval = val;
    
    if (yloc == discr[1]-1) {
        xedge[yloc+1][zloc][1] = vertlist[10];
    }
    if (zloc == discr[2]-1) {
        xedge[yloc][zloc+1][0] = vertlist[5];
        yedge[zloc+1][0] = vertlist[6];
    }
   
   return(1);
}

/*
   Linearly interpolate the position where an isosurface cuts
   an edge between two vertices, each with their own scalar value
*/
       
double[] VertexInterp(double isolevel,double[] p1, double[] p2, double valp1,double valp2)
{
   double mu;
   double[] p;
   p = new double[3];

   if (Math.abs(isolevel-valp1) < 0.00001)
      return(convertToXYZ(p1));
   if (Math.abs(isolevel-valp2) < 0.00001)
      return(convertToXYZ(p2));
   if (Math.abs(valp1-valp2) < 0.00001)
      return(convertToXYZ(p1));
   mu = (isolevel - valp1) / (valp2 - valp1);
   p[0] = p1[0] + mu * (p2[0] - p1[0]);
   p[1] = p1[1] + mu * (p2[1] - p1[1]);
   p[2] = p1[2] + mu * (p2[2] - p1[2]);

   return(convertToXYZ(p));
}/*
// Bisection method;  not working yet
double[] VertexInterp(double isolevel,double[] pt1, double[] pt2, double valp1,double valp2)
{
   double[] p,p1,p2;
   p = new double[3];
   p1 = new double[3];
   p2 = new double[3];
   p1[0] = pt1[0];
   p1[1] = pt1[1];
   p1[2] = pt1[2];
   p2[0] = pt2[0];
   p2[1] = pt2[1];
   p2[2] = pt2[2];
 
   double size = (p2[0]-p1[0])*(p2[0]-p1[0]) + (p2[1]-p1[1])*(p2[1]-p1[1]) + (p2[2]-p1[2])*(p2[2]-p1[2]);
   
   double pval;
   int i=0;

  
   p[0] = (p2[0]+p1[0])/2.;
   p[1] = (p2[1]+p1[1])/2.;
   p[2] = (p2[2]+p1[2])/2.;
   pval = m_func2.eval(0,p);
   
   while ((p2[0]-p1[0])*(p2[0]-p1[0]) + (p2[1]-p1[1])*(p2[1]-p1[1]) + (p2[2]-p1[2])*(p2[2]-p1[2]) > .01*size) {
       
       if (((pval < isolevel) && (valp1 < isolevel)) || ((pval > isolevel) && (valp1 > isolevel))) {   //p and p1 are same sign, so move p1 to p
            
           p1[0]=p[0];
           p1[1]=p[1];
           p1[2]=p[2];
           valp1 = pval;

       } else {
           
           p2[0]=p[0];
           p2[1]=p[1];
           p2[2]=p[2];
           valp2 = pval;
       }
       p[0] = (p2[0]+p1[0])/2.;
       p[1] = (p2[1]+p1[1])/2.;
       p[2] = (p2[2]+p1[2])/2.;
       pval = m_func2.eval(0,p);
       
       
       i++;
       if (i>10) { 
           
           break;
       }
       
   }
 //  System.out.println(i + " "+p[0] +" "+p[1]+" "+p[2]);
   return(p);
}
*/
void initXstorage (int[] discr,double xmin, double ymin, double zmin, double deltaX, double deltaY, double deltaZ, double isolevel) {
            double[][] cubepoints;
            cubepoints = new double[8][3];
            double[] cubevals;
            cubevals = new double[8];
            double x, y, z;
            
            x = xmin;
            y = ymin;
            for (int j=0; j < discr[1]+1; j++) {
                    y = ymin + deltaY*j;
                    z = zmin;
                    for (int k=0; k < discr[2]+1; k++) {
                        z = zmin + deltaZ*k;
                        //do edges 3 and 8 -> xedge
                        cubepoints[0]= new double[] {x,y,z};
                        cubepoints[3]= new double[] {x,y+deltaY,z};
                        cubepoints[4]= new double[] {x,y,z+deltaZ};
                        cubepoints[7]= new double[] {x,y+deltaY,z+deltaZ};
                        cubevals[0] = m_func2.eval(0,cubepoints[0]);
                        cubevals[3] = m_func2.eval(0,cubepoints[3]);
                        cubevals[4] = m_func2.eval(0,cubepoints[4]);
                        cubevals[7] = m_func2.eval(0,cubepoints[7]);
                        if (((cubevals[0]<isolevel) != (cubevals[3] < isolevel))) {
                            xedge[j][k][0] =
                            addVertex(new PdVector( VertexInterp(isolevel,cubepoints[3],cubepoints[0],cubevals[3],cubevals[0])));
                        } else { xedge[j][k][0] = -1;}
                        if (((cubevals[0]<isolevel) != (cubevals[4] < isolevel))) {
                            xedge[j][k][1] =
                            addVertex(new PdVector( VertexInterp(isolevel,cubepoints[0],cubepoints[4],cubevals[0],cubevals[4])));
                        } else { xedge[j][k][1] = -1;}
                        xpoint[j][k] = cubepoints[7];
                        xval[j][k] = cubevals[7];
                    }
            }
}    

void initYstorage(int[] discr,double x, double ymin, double zmin, double deltaX, double deltaY, double deltaZ, double isolevel) {
            double y, z;
            double[][] cubepoints;
            cubepoints = new double[8][3];
            double[] cubevals;
            cubevals = new double[8];
            
            y = ymin;
            z = zmin;
            for (int k=0; k < discr[2]+1; k++) {
                        z = zmin + deltaZ*k;
                        //do edges 0 and 9 -> yedge
                        cubepoints[0]= new double[] {x,y,z};
                        cubepoints[1]= new double[] {x+deltaX,y,z};
                        cubepoints[5]= new double[] {x+deltaX,y,z+deltaZ};
                        cubevals[0] = m_func2.eval(0,cubepoints[0]);
                        cubevals[1] = m_func2.eval(0,cubepoints[1]);
                        cubevals[5] = m_func2.eval(0,cubepoints[5]);
                        if ((cubevals[0]<isolevel) != (cubevals[1] < isolevel)) {
                            yedge[k][0] =
                            addVertex(new PdVector( VertexInterp(isolevel,cubepoints[0],cubepoints[1],cubevals[0],cubevals[1])));
                        } else {yedge[k][0] = -1;}
                        if (((cubevals[1]<isolevel) != (cubevals[5] < isolevel))) {
                            yedge[k][1] =
                            addVertex(new PdVector( VertexInterp(isolevel,cubepoints[1],cubepoints[5],cubevals[1],cubevals[5])));
                        } else {yedge[k][1] = -1;}
                        ypoint[k] = cubepoints[5];
                        yval[k] = cubevals[5];
  
            }
}

void initZstorage (int[] discr,double x, double y, double zmin, double deltaX, double deltaY, double deltaZ, double isolevel) {
    double z = zmin;
     double[][] cubepoints;
            cubepoints = new double[8][3];
            double[] cubevals;
            cubevals = new double[8];
    
    //do edges 1 and 2 -> zedge
                        cubepoints[1]= new double[] {x+deltaX,y,z};
                        cubepoints[2]= new double[] {x+deltaX,y+deltaY,z};
                        cubepoints[3]= new double[] {x,y+deltaY,z};
                        cubevals[1] = m_func2.eval(0,cubepoints[1]);
                        cubevals[2] = m_func2.eval(0,cubepoints[2]);
                        cubevals[3] = m_func2.eval(0,cubepoints[3]);
                        if ((cubevals[1]<isolevel) != (cubevals[2] < isolevel)) {
                            zedge[0] =   //stores cubeedge 1
                            addVertex(new PdVector( VertexInterp(isolevel,cubepoints[1],cubepoints[2],cubevals[1],cubevals[2])));
                        } else {zedge[0] = -1;}
                        if ((cubevals[2]<isolevel) != (cubevals[3] < isolevel)) {
                            zedge[1] =  //stores cubeedge 2
                            addVertex(new PdVector( VertexInterp(isolevel,cubepoints[2],cubepoints[3],cubevals[2],cubevals[3])));
                        } else {zedge[1] = -1;}
                        zpoint = cubepoints[2];
                        zval = cubevals[2];
}

public void setEqn(String func) {
            int ind = func.indexOf("=");
            String out = func.substring(0,ind) + "- (" + func.substring(ind+1) + ")";
            if (!m_func2.getExpression(0).equals(out)) {
                m_func2.setExpression(0, out);
                update(this);
            }
}

 public void setDomain(double[] domain) {
            if (m_bUseGlobalDomain) {
                m_domain.setVals(domain);
                update(this);
            } else {
                globaldomain = domain;
            }
 }

 public double[] convertToXYZ(double[] abc) {
    if (coords == TYPE_IMPSURF) { 
        return abc;
    } else if (coords == TYPE_IMPPOLAR) {
        double[] xyz = new double[3];
        xyz[0] = abc[0]*Math.cos(abc[1]);
        xyz[1] = abc[0]*Math.sin(abc[1]);
        xyz[2] = abc[2];
        return xyz;
    } 
    
    return abc;
}

 
}
