/*
 * PgVectorField.java
 *
 * Created on December 5, 2004, 6:20 PM
 */

package Grapher2D.geom;

import java.awt.Color;
import jv.function.PuFunction;
import jv.number.PuDouble;
import jv.object.PsPanel;
import jv.geom.PgPointSet;
import jv.number.PdColor;
import jv.number.PuInteger;
import jv.object.PsDebug;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
import jv.geom.PgVectorField;
import Grapher2D.util.PuDomain;
/**
 *
 * @author David
 */

public class PgVecFld extends PgPointSet {
    protected       PuFunction                      m_func2;
    protected       PuDomain                        m_domain;
    protected       PgVectorField                   m_vf;
    protected       boolean                         m_bUseGlobalDomain = false;
    protected       double[]                        globaldomain;
    /** Creates a new instance of PgVectorField */
    public PgVecFld(String Pname) {
        super(2);
        if (getClass() == PgVecFld.class) {
            init(Pname);
        }
    }
    
    public void init(String Pname) {
        super.init();
        setName(Pname);
        
        m_vf = new PgVectorField(2, PgVectorField.VERTEX_BASED);
        
        m_func2 = new PuFunction(2,2);
        m_func2.setName("Functions");
        m_func2.setVariables(new String[] {"x","y"});
        m_func2.setExpressionNames(new String[] {"dx","dy"});
        m_func2.setExpressions(new String[] {"-y","x"});
        m_func2.setParent(this);
        
        m_domain = new PuDomain();
        m_domain.setVars(new String[] {"x","y"});
        m_domain.setDefVals(new double[] {-5.0,5.0});
        m_domain.setDefDiscr(new int[] {10});
        m_domain.setAllowSlices(true);
        m_domain.setIsVF(true);
        m_domain.init();
        m_domain.setParent(this);
       
//        assureVertexColors();
//        showVertexColors(true);
        showVertices(false);
        m_vf.assureVectorColors();
                    m_vf.showIndividualMaterial(true);
                    m_vf.showVectorArrows(true);
                    m_vf.setGlobalVectorColor(Color.black);
        
        
        
        
        computeVf();
        
        m_vf.setGeometry(this);
        addVectorField(m_vf);
        
        
        
    }
    
    public void start() {
		if (PsDebug.NOTIFY) PsDebug.notify("start: ");
    }

    public boolean update(Object event) {
		if (event == this) {
                        computeVf();
			return super.update(this);
		} else if (event == m_domain) {
                    update(this);
                    
                } else if (event == m_func2) {
                    update(this);
                }
                return true;
    }
    
    public void computeVf() {
 
            int[] discr;
            double[] delta;
            delta = new double[2];
            discr = m_domain.getDiscr();
            
            
            for (int i=0; i<2;i++) {
                if (m_domain.isSliced(i)) {
                    discr[i] = 1;
                    delta[i] = .1;
                } else {
                    delta[i] = (m_domain.getMax(i)-m_domain.getMin(i))/(discr[i]-1.);
                }
            }
            
            int numVertices = discr[0]*discr[1];
            setNumVertices(numVertices);
            m_vf.setNumVectors(numVertices);
            
            int i,j;
            double u,v;
            
            int ind = 0;
  
                for (i=0; i<discr[0];i++){
                    for (j=0; j<discr[1];j++){
                        
                                u = m_domain.getMin(0)+delta[0]*i;
                                v = m_domain.getMin(1)+delta[1]*j;
                                
                                double[] inp = {u,v};
				double x,y;
                                x = m_func2.eval(0,inp);
                                y = m_func2.eval(1,inp);
                               
                                
                                double scl;
                                if (m_domain.isSclVecs()) {
                                    double length = Math.sqrt(x*x+y*y);
                                    scl = m_domain.getVeclength()/(length+0.1);
                                } else {
                                    scl = m_domain.getVeclength()/10.;
                                }
 //                               double length = Math.sqrt(x*x+y*y+z*z);
 //                               double scl = Math.min(deltaU,Math.min(deltaV,deltaW))/(length+0.1);
                      
                                setVertex(ind,u,v);
                                m_vf.setVector(ind,x*scl,y*scl);
                                ind++;
                        
                    }
                }
         
        }
    public void setDomain(double[] domain) {
            if (m_bUseGlobalDomain) {
                m_domain.setVals(domain);
                update(this);
            } else {
                globaldomain = domain;
            }
        }
}
